import java.util.*;
import java.io.*;
import javax.swing.JOptionPane;
import com.apple.eawt.*;

/*-
 * Copyright (C) 2007 Erik Larsson
 * 
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

public class Launcher {
    public static void main(String[] args) {
	Application thisApp = Application.getApplication();
	thisApp.setEnabledPreferencesMenu(false);
	thisApp.setEnabledAboutMenu(true);
	final MountListWindow mlWindow = new MountListWindow();
	mlWindow.setVisible(true);
	
	thisApp.addApplicationListener(mlWindow);
    }

    /**
     * @return 0 if successful, 2 if the folder could not opened in Finder, -1 if the mount did not succeed
     */
    public static int mount(File rarFile, File[] outMountDir) throws Exception {
	if(rarFile.exists() && rarFile.canRead()) {
	    System.out.println("mount:");
	    System.out.println("  rarFile: " + rarFile.getAbsolutePath());
	    /* Create a directory for the mount point. */
	    String rarFilename = rarFile.getName();
	    String mountDirBaseName = rarFilename;// rarFilename.substring(0, rarFilename.lastIndexOf("."));
	    String mountDirName = mountDirBaseName;
	    String baseMountDir = "/Volumes/";
	    File mountDir = new File(baseMountDir, mountDirName);
	    int counter = 1;
	    while(mountDir.exists()) {
		mountDirName = mountDirBaseName + "-" + counter++;
		mountDir = new File(baseMountDir, mountDirName);
	    }
	    mountDir.mkdir();
	    System.out.println("  mountdir: " + mountDir.getAbsolutePath());
	    
	    File newWorkingDir = new File(System.getProperty("user.dir")).getParentFile();

	    String[] cmdarray = { newWorkingDir.getAbsolutePath() + "/rarfs_fuse", rarFile.getAbsolutePath(), "-oping_diskarb", "-ovolname=" + mountDirName, mountDir.getAbsolutePath() };
	    Process p = Runtime.getRuntime().exec(cmdarray, null, newWorkingDir);
	    p.waitFor();
	    System.out.println("  rarfs mount exited with value " + p.exitValue());
	    if(p.exitValue() == 0) {
		p = Runtime.getRuntime().exec("open " + mountDir.getAbsolutePath());
		p.waitFor();
		System.out.println("  open exited with value " + p.exitValue());
		//JOptionPane.showMessageDialog(null, "Mounted!", "Info", JOptionPane.INFORMATION_MESSAGE);
		outMountDir[0] = mountDir;
		if(p.exitValue() == 0)
		    return 0;
		else
		    return 2;
	    }
	    else
		return -1;
	}
	else
	    return -1;
    }

    public static int unmount(Mount m, boolean force) throws Exception {
	String command = "umount";
 	if(force)
	    command += " -f";
	command += " " + m.mountDir.getAbsolutePath();
	
	Process p = Runtime.getRuntime().exec(command);
	p.waitFor();
	if(p.exitValue() == 0) {
	    if(m.mountDir.delete())
		return 0;
	    else
		return -1;
	}
	else
	    return -2;
    }
    
    public static void oldmain(String[] args) throws Exception {
	//JOptionPane.showMessageDialog(null, "Launcher launching with args.length=" + args.length, "Info", JOptionPane.INFORMATION_MESSAGE);
	if(args.length < 1)
	    return;
	
	String rarPathname = args[0];
	JOptionPane.showMessageDialog(null, "rarPathname=" + rarPathname, "Info", JOptionPane.INFORMATION_MESSAGE);
	File rarFile = new File(rarPathname);
	if(rarFile.exists() && rarFile.canRead()) {
	    /* Create a directory for the mount point. */
	    String rarFilename = rarFile.getName();
	    String baseMountDir = "/Volumes/" + rarFilename.substring(0, rarFilename.lastIndexOf("."));
	    File mountDir = new File(baseMountDir);
	    int counter = 1;
	    while(mountDir.exists()) {
		mountDir = new File(baseMountDir + "-" + counter++);
	    }
	    mountDir.mkdir();
	    System.out.println("mountdir: " + mountDir);
	    
	    // Adjust the environment
	    Set<Map.Entry<String, String>> envset = System.getenv().entrySet();
	    String[] environment = new String[envset.size()];
	    int i = 0;
	    File newWorkingDir = new File(System.getProperty("user.dir")).getParentFile();
	    JOptionPane.showMessageDialog(null, "newWorkingDir=" + newWorkingDir, "Info", JOptionPane.INFORMATION_MESSAGE);
// 	    for(Map.Entry<String, String> entry : envset) {
// 		System.out.println("\"" + entry.getKey() + "\" = \"" + entry.getValue() + "\"");
// 		if(entry.getKey().equals("PWD")) {
// 		    newWorkingDir = new File(entry.getValue()).getParentFile();
// 		    environment[i++] = "PWD=" + newWorkingDir.getAbsolutePath();
// 		    System.out.println("  NEW PWD: " + environment[i-1]);
// 		}
// 		else
// 		    environment[i++] = entry.getKey() + "=" + entry.getValue();
// 	    }
	    //System.getenv("PWD"
	    String[] cmdarray = { newWorkingDir.getAbsolutePath() + "/rarfs_fuse", rarFile.getAbsolutePath(), mountDir.getAbsolutePath() };
	    Process p = Runtime.getRuntime().exec(cmdarray, null, newWorkingDir);
	    p.waitFor();
	    System.out.println("Mount exited with value " + p.exitValue());
	    if(p.exitValue() == 0) {
		p = Runtime.getRuntime().exec("open " + mountDir.getAbsolutePath());
		p.waitFor();
		System.out.println("open exited with value " + p.exitValue());
		JOptionPane.showMessageDialog(null, "Mounted!", "Info", JOptionPane.INFORMATION_MESSAGE);
	    }
	}
	else
	    System.out.println("RAR file can not be read...");
    }
}